	-----------------------------------------------
    --       Complex Variable Math Library       --
	--     (c) Gerardo V. Lozada, PEE - 1997     --
	-----------------------------------------------

include math.e

global constant cinit={0.0,0.0}

global function cabs(sequence x)
	return sqrt(power(x[1],2)+power(x[2],2))
end function

global function arctan2(atom x,atom y)
	atom z
	if abs(x)>0 then
		z=rad_deg(abs(arctan(y/x)))
		if x>0.0 then
			if y<0.0 then
				return -z
			else
				return z
			end if
		else
			if y<0.0 then
				return -180.0+z
			else
				return 180.0-z
			end if
		end if
	else
		if y=0.0 then
			return 0.0
		else
			if y>0.0 then
				return 90.0
			else
				return -90.0
			end if
		end if
	end if
end function

global function arg(sequence z)
	return arctan2(z[1],z[2])
end function

global function real(sequence z)
	return z[1]
end function

global function imag(sequence z)
	return z[2]
end function

global function conj(sequence z)
	return {real(z),-imag(z)}
end function

global function rpc(sequence z)
	return {cabs(z),arg(z)}
end function

global function prc(sequence z)
	atom zr,zi
		zr=z[1]*cos(deg_rad(z[2]))
		zi=z[1]*sin(deg_rad(z[2]))
		return {zr,zi}
end function

global function cmult(sequence a,sequence b)
	sequence c
	c=cinit
	c[1]=a[1]*b[1]-a[2]*b[2]
	c[2]=a[1]*b[2]+a[2]*b[1]
	return c
end function

global function cdiv(sequence a,sequence b)
	sequence c
	c=cinit
	if cabs(b)>0 then
		c[1]=a[1]*b[1]+a[2]*b[2]
		c[2]=a[2]*b[1]-a[1]*b[2]
		c=c/(power(b[1],2)+power(b[2],2))
	end if
	return c
end function

global function cpower(sequence z,atom n)
	atom zm,za
	zm=power(cabs(z),n)
	za=arg(z)
	if za<0.0 then
		za=360.0+za
	end if
	za=za*n
	za=mod(za,360.0)
	return prc({zm,za})
end function

global function cdeterminant(sequence b)
	sequence a,aprev,determ,dummy
	integer n
	n=length(b)
	dummy=repeat(cinit,n)
	a=b
	aprev=b
	for k=1  to n by 1 do
		for i=k+1  to n by 1 do
			for j=1  to k by 1 do
				a[i][j]=0.0
			end for
			for j=k+1  to n by 1 do
				a[i][j]=aprev[i][j]-cdiv(cmult(aprev[k][j],aprev[i][k]),aprev[k][k])
			end for
		end for
		aprev=a
	end for
	determ={1.0,0.0}
	for i=1  to n by 1 do
		determ=cmult(determ,a[i][i])
	end for
	return determ
end function

global function cinverse(sequence c)
	sequence a,b,temp,dummy
	integer itemp,n
	n=length(c)
	dummy=repeat(cinit,n)
	b=repeat(dummy,n)
	a=repeat(dummy,n)
	for i=1  to n do
		for j=1  to n by 1 do
			b[i][j]=cinit
			if i=j then
				b[i][i]={1.0,0.0}
			end if
			 a[i][j]=c[i][j]
		end for
	end for
	for i=1  to n by 1 do
		itemp=i
		temp=a[i][i]
		for j=i  to n by 1 do
			if (cabs(a[j][i])>cabs(temp)) then
				temp=a[j][i]
				itemp=j
			end if
		end for
		if itemp>i then
			for j=1  to n by 1 do
				temp=a[itemp][j]
				a[itemp][j]=a[i][j]
				a[i][j]=temp
				temp=b[itemp][j]
				b[itemp][j]=b[i][j]
				b[i][j]=temp
			end for
		end if
		temp=a[i][i]
		if cabs(temp)=0 then
			printf(1,"\nError: Singular Matrix\n",{})
			abort(1)
		end if
		for j=1  to n by 1 do
			a[i][j]=cdiv(a[i][j],temp)
			b[i][j]=cdiv(b[i][j],temp)
		end for
		for j=1  to n by 1  do
			if ((j<i) or (j>i)) then
				temp=a[j][i]
				for k=1  to n by 1 do
					a[j][k]=a[j][k]-cmult(temp,a[i][k])
					b[j][k]=b[j][k]-cmult(temp,b[i][k])
				end for
			end if
		end for
	end for
	return b
end function

global function cmultiply_matrix(sequence a,sequence b)
	sequence c,dummy
	integer nra,nca,nrb,ncb
	nra=length(a)
	nca=length(a[1])
	nrb=length(b)
	ncb=length(b[1])
	if nca=nrb then
	    dummy=repeat(cinit,ncb)
	    c=repeat(dummy,nra)
	    for i=1 to nra do
			for j=1 to ncb do
				for k=1 to nca do
					c[i][j]=c[i][j]+cmult(a[i][k],b[k][j])
				end for
			end for
		end for
	else
	    printf(1,"\nMatrix Multiplication Error: #Rows1 <> #Columns2\n",{})
	    abort(1)
	end if
	return c
end function

global function ctranspose(sequence a)
	integer n
	sequence b
	n=length(a)
	b=repeat(repeat(cinit,n),1)
	for i=1 to n do
		b[1][i]=a[i][1]
	end for
	return b
end function

global function cexp(sequence z)
	atom x,y
	x=exp(z[1])*cos(z[2])
	y=exp(z[1])*sin(z[2])
	return {x,y}
end function

global function ccos(sequence z)
	atom x,y
	x=(exp(z[2])+exp(-z[2]))/2.0*cos(z[1])
	y=(exp(z[2])-exp(-z[2]))/2.0*sin(z[1])
	return {x,y}
end function

global function csin(sequence z)
	atom x,y
	x=(exp(z[2])+exp(-z[2]))/2.0*sin(z[1])
	y=(exp(z[2])-exp(-z[2]))/2.0*cos(z[1])
	return {x,y}
end function

global function ctan(sequence z)
	return cdiv(csin(z),ccos(z))
end function

global function ccosh(sequence z)
	atom x,y
	x=cos(z[2])*cosh(z[1])
	y=sin(z[2])*sinh(z[1])
	return {x,y}
end function

global function csinh(sequence z)
	atom x,y
	x=cos(z[2])*sinh(z[1])
	y=sin(z[2])*cosh(z[1])
	return {x,y}
end function

global function ctanh(sequence z)
	return cdiv(csinh(z),ccosh(z))
end function
