-- Short Circuit Fault Currents and Voltages
-- Gerardo V. Lozada, M.S., P.E.E. - 2003

with trace

include get_enh.e
include complex.e
include wildcard.e
include graphics.e

sequence a,a2,amatrix,cunity,v

procedure print_matrix(sequence a,sequence title,integer fno)
	integer nr,nc
	nr=length(a)
	nc=length(a[1])
	printf(1,"\n%s\n\n",{title})
	printf(fno,"\n%s\n\n",{title})
	for i=1 to nr do
		for j=1 to nc do
			printf(1,"%11.4e ",{a[i][j][1]})
			printf(fno,"%11.4e ",{a[i][j][1]})
			if a[i][j][2]<0.0 then
				printf(1,"-j%11.4e  ",{abs(a[i][j][2])})
				printf(fno,"-j%11.4e  ",{abs(a[i][j][2])})
			else
				printf(1,"+j%11.4e  ",{a[i][j][2]})
				printf(fno,"+j%11.4e  ",{a[i][j][2]})
			end if
		end for
		printf(1,"\n",{})
		printf(fno,"\n",{})
	end for
	printf(1,"\n",{})
	printf(fno,"\n",{})
end procedure

function addbus(sequence bus,integer bn)
	bus=append(bus,bn)
	return bus
end function

function newbus(sequence bus,integer bus1)
	integer n,new,ii
	n=length(bus)
	if bus1=0 then
		return 0
	end if
	new=1
	if n>0 then
		ii=1
		while (ii<=n) and (new=1) do
			if bus1=bus[ii] then
				new=0
			end if
			ii=ii+1
		end while
	end if
	return new
end function

function addbustype1(sequence zbus,atom r,atom x)
	integer n
	zbus=append(zbus,repeat(cinit,length(zbus)))
	n=length(zbus)
	for i=1 to n-1 do
		zbus[i]=append(zbus[i],cinit)
	end for
	zbus[n]=append(zbus[n],{r,x})
	return zbus
end function

function addbustype2(sequence zbus,sequence bus,integer oldbus,atom r,atom x)
	integer n,ii,iold
	n=length(bus)
	ii=1
	iold=0
	while (iold=0) and (ii<=n) do
		if (oldbus=bus[ii]) then
			iold=ii
		end if
		ii=ii+1
	end while
	zbus=append(zbus,repeat(cinit,length(zbus)))
	n=length(zbus)
	for i=1 to n do
		zbus[i]=append(zbus[i],zbus[i][iold])
	end for
	for i=1 to n-1 do
		zbus[n][i]=zbus[iold][i]
	end for
	zbus[n][n]=zbus[iold][iold]+{r,x}
	return zbus
end function

function addbustype3(sequence zbus,sequence bus,integer oldbus,atom r,atom x)
	integer n,ii,iold
	sequence zcol,zcolt,ztemp,znn
	n=length(bus)
	zcol=repeat({cinit},n)
	ii=1
	iold=0
	while (iold=0) and (ii<=n) do
		if (oldbus=bus[ii]) then
			iold=ii
		end if
		ii=ii+1
	end while
	for i=1 to n do
		zcol[i][1]=zbus[i][iold]
	end for
	zcolt=ctranspose(zcol)
	znn=zbus[iold][iold]+{r,x}
	ztemp=cmultiply_matrix(zcol,zcolt)
	for i=1 to n do
		for j=1 to n do
			ztemp[i][j]=cdiv(ztemp[i][j],znn)
		end for
	end for
	zbus=zbus-ztemp
	return zbus
end function

function addbustype4(sequence zbus,sequence bus,integer frombus,integer tobus,atom r,atom x)
	integer n,ii,ifrom,ito
	sequence zcol,zcolt,ztemp,znn
	n=length(bus)
	zcol=repeat({cinit},n)
	ii=1
	ifrom=0
	ito=0
	while ((ifrom=0) or (ito=0)) and (ii<=n) do
		if (frombus=bus[ii]) then
			ifrom=ii
		end if
		if (tobus=bus[ii]) then
			ito=ii
		end if
		ii=ii+1
	end while
	for i=1 to n do
		zcol[i][1]=zbus[i][ifrom]-zbus[i][ito]
	end for
	zcolt=ctranspose(zcol)
	znn=zbus[ifrom][ifrom]+zbus[ito][ito]+{r,x}-2.0*zbus[ifrom][ito]
	ztemp=cmultiply_matrix(zcol,zcolt)
	for i=1 to n do
		for j=1 to n do
			ztemp[i][j]=cdiv(ztemp[i][j],znn)
		end for
	end for
	zbus=zbus-ztemp
	return zbus
end function

function abc_current_voltage(sequence i012,sequence zbus1,sequence zbus2,sequence zbus0,sequence v,integer ifaultedbus)
	sequence iabc,va0,va1,va2,v012,vabc,ia0,ia1,ia2
	ia0=i012[1][1]
	ia1=i012[2][1]
	ia2=i012[3][1]
	iabc=cmultiply_matrix(amatrix,i012)
	va0=-cmult(ia0,zbus0[ifaultedbus][ifaultedbus])
	va1=v-cmult(ia1,zbus1[ifaultedbus][ifaultedbus])
	va2=-cmult(ia2,zbus2[ifaultedbus][ifaultedbus])
	v012={{va0},{va1},{va2}}
	vabc=cmultiply_matrix(amatrix,v012)
	return {iabc,v012,vabc}
end function

function adjacent(sequence vertex,sequence edge)
	integer m,n,ii,identical
	sequence adj
	n=length(vertex)
	m=length(edge)
	adj=repeat({},n)
	for i=1 to n do
		for j=1 to n do
			for k=1 to m do
				if (((vertex[i]=edge[k][1])and(vertex[j]=edge[k][2]))or((vertex[j]=edge[k][1])and(vertex[i]=edge[k][2]))) then
					if (length(adj[i])=0) then
						adj[i]={vertex[j]}
					else
						identical=0
						ii=1
						while ((ii<=length(adj[i]))and(identical=0)) do
							if (adj[i][ii]=vertex[j]) then
								identical=1
							end if
							ii=ii+1
						end while
						if (identical=0) then
							adj[i]=adj[i]&{vertex[j]}
						end if
					end if
				end if
			end for
		end for
	end for
	return adj
end function

function findpath(integer x,integer y,sequence bus,sequence adj)
	integer nbus,nadj,ii,jj,kk,ll,ix,iy,foundx,foundy,foundz,curr_bus
	sequence t,prev_adj,adjred,path
	nbus=length(bus)
	t=repeat({0,0},nbus)
	ii=0
	foundx=0
	foundy=0
	while (ii<nbus) and ((foundx=0) or (foundy=0)) do
		ii+=1
		if (bus[ii]=x) then
			ix=ii
			foundx=1
		else
			if (bus[ii]=y) then
				iy=ii
				foundy=1
			end if
		end if
	end while
	prev_adj={x}
	foundy=0
	t[ix][1]=0
	while (foundy=0) do
		ii=0
		while (ii<length(prev_adj)) do
			ii+=1
			jj=0
			foundx=0
			while (jj<nbus) and (foundy=0) do
				jj+=1
				if (bus[jj]=prev_adj[ii]) then
					kk=0
					nadj=length(adj[jj])
					foundx=1
					while (kk<nadj) and (foundy=0) do
						kk+=1
						ll=0
						foundz=0
						while (ll<nbus) do
							ll+=1
							if ((bus[ll]=adj[jj][kk]) and (t[ll][1]=0)) then
								foundz=1
								t[ll][1]=1
								t[ll][2]=bus[jj]
								if (bus[ll]=y) then
									foundy=1
								end if
							end if
						end while
					end while
					adjred=adj[jj]
					prev_adj=prev_adj&adjred
				end if
			end while
		end while
	end while
	path={y}
	curr_bus=y
	ii=0
	foundx=0
	while (ii<length(t)) and (curr_bus!=x) do
		ii+=1
		if (bus[ii]=curr_bus) then
			curr_bus=t[ii][2]
			path=path&curr_bus
			ii=0
		end if
	end while
	return path
end function

procedure	fault_contributions(sequence iabc,sequence vabc,sequence v,sequence zbus1,sequence zbus2,sequence zbus0,sequence bus,sequence rawline,integer ifaultedbus,integer fno,atom bmva)
	sequence i012,v012,ia0,ia1,ia2,va0,va1,va2,va0o,va1o,va2o,vabco,v012o,v012_all,iabco,i012o,imax,ia,ib,ic,phase_shift,path,edge,adj
	integer nbus,nline,ii,found
	atom fcon,ps_angle
	i012=cmultiply_matrix(cinverse(amatrix),iabc)
	ia0=i012[1][1]
	ia1=i012[2][1]
	ia2=i012[3][1]
	v012=cmultiply_matrix(cinverse(amatrix),vabc)
	va0=v012[1][1]
	va1=v012[2][1]
	va2=v012[3][1]
	nbus=length(bus)
	nline=length(rawline)
	v012_all={}
	edge={}
	for i=1 to nline do
		edge=edge&{{rawline[i][4],rawline[i][5]}}
	end for
	adj=adjacent(bus,edge)
	printf(1,"\nP.U. Voltages In Other Buses:\n\n",{})
	printf(fno,"\nP.U. Voltages In Other Buses:\n\n",{})
	for i=1 to nbus do
		if (i!=ifaultedbus) then
			printf(1,"\nBus %d:\n",{bus[i]})
			printf(fno,"\nBus %d:\n",{bus[i]})
			phase_shift=cinit
			path=findpath(bus[ifaultedbus],bus[i],bus,adj)
			ps_angle=0.0
			for j=1 to length(path)-1 do
				ii=0
				found=0
				while (ii<nline) and (found=0) do
					ii+=1
					if ((path[j]=rawline[ii][4]) and (path[j+1]=rawline[ii][5])) then
						ps_angle+=rawline[ii][7]
						found=1
					else
						if ((path[j]=rawline[ii][5]) and (path[j+1]=rawline[ii][4])) then
							ps_angle-=rawline[ii][7]
							found=1
						end if
					end if
				end while
			end for
			printf(1,"\nNet Phase Shift: %7.1f Degrees\n",{ps_angle})
			printf(fno,"\nNet Phase Shift: %7.1f Degrees\n",{ps_angle})
			phase_shift={cos(deg_rad(ps_angle)),sin(deg_rad(ps_angle))}
			va0o=-cmult(ia0,zbus0[i][ifaultedbus])
			va1o=cmult(v-cmult(ia1,zbus1[i][ifaultedbus]),phase_shift)
			va2o=cmult(-cmult(ia2,zbus2[i][ifaultedbus]),conj(phase_shift))
			v012o={{va0o},{va1o},{va2o}}
			vabco=cmultiply_matrix(amatrix,v012o)
			printf(1,"P.U. Sequence Voltages: a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0o),arg(va0o),cabs(va1o),arg(va1o),cabs(va2o),arg(va2o)})
			printf(1,"P.U. Phase Voltages: ",{})
			printf(1,"A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(vabco[1][1]),arg(vabco[1][1]),cabs(vabco[2][1]),arg(vabco[2][1]),cabs(vabco[3][1]),arg(vabco[3][1])})
			printf(fno,"P.U. Sequence Voltages: a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0o),arg(va0o),cabs(va1o),arg(va1o),cabs(va2o),arg(va2o)})
			printf(fno,"P.U. Phase Voltages: ",{})
			printf(fno,"A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(vabco[1][1]),arg(vabco[1][1]),cabs(vabco[2][1]),arg(vabco[2][1]),cabs(vabco[3][1]),arg(vabco[3][1])})
			va1o=cmult(va1o,conj(phase_shift))
			va2o=cmult(va2o,phase_shift)
			v012o={{va0o},{va1o},{va2o}}
			v012_all=v012_all&{v012o}
		else
			v012_all=v012_all&{v012}
		end if
	end for
	printf(1,"\n\nFault Contributions from Adjacent Buses:\n\n",{})
	printf(fno,"\n\nFault Contributions from Adjacent Buses:\n\n",{})
	for i=1 to nbus do
		if (i!=ifaultedbus) then
			for j=1 to nline do
				if ((rawline[j][4]=bus[ifaultedbus]) and (rawline[j][5]=bus[i])) or ((rawline[j][4]=bus[i]) and (rawline[j][5]=bus[ifaultedbus])) then
					printf(1,"\nFrom Bus %d:\n",{bus[i]})
					printf(fno,"\nFrom Bus %d:\n",{bus[i]})
					ia0=cinit
					phase_shift=cinit
					path=findpath(bus[ifaultedbus],bus[i],bus,adj)
					ps_angle=0.0
					for k=1 to length(path)-1 do
						ii=0
						found=0
						while (ii<nline) and (found=0) do
							ii+=1
							if ((path[k]=rawline[ii][4]) and (path[k+1]=rawline[ii][5])) then
								if ((rawline[ii][6]=0) or (rawline[ii][6]=1)) then
									ia0=cdiv(v012_all[i][1][1]-v012[1][1],rawline[ii][3])
								end if
								ps_angle+=rawline[ii][6]
								found=1
							else
								if ((path[k]=rawline[ii][5]) and (path[k+1]=rawline[ii][4])) then
									ps_angle-=rawline[ii][7]
									found=1
								end if
							end if
						end while
					end for
					printf(1,"\nNet Phase Shift: %7.1f\n",{ps_angle})
					printf(fno,"\nNet Phase Shift: %7.1f\n",{ps_angle})
					phase_shift={cos(deg_rad(ps_angle)),sin(deg_rad(ps_angle))}
					ia1=cmult(cdiv(v012_all[i][2][1]-v012[2][1],rawline[j][1]),phase_shift)
					ia2=cmult(cdiv(v012_all[i][3][1]-v012[3][1],rawline[j][2]),conj(phase_shift))
					i012o={{ia0},{ia1},{ia2}}
					iabco=cmultiply_matrix(amatrix,i012o)
					ia=iabco[1][1]
					ib=iabco[2][1]
					ic=iabco[3][1]
					imax=ia
					if (cabs(ib)>cabs(imax)) then
						imax=ib
					end if
					if (cabs(ic)>cabs(imax)) then
						imax=ic
					end if
					fcon=bmva*cabs(cmult(v,conj(imax)))
					printf(1,"Fault Duty Contribution: %10.1f MVA, Ia0: %6.4f p.u. @ %6.1f Deg., ",{fcon,cabs(ia0),arg(ia0)})
					printf(1,"Ia1: %6.4f p.u. @ %6.1f Deg., Ia2: %6.4f p.u. @ %6.1f Deg.\n",{cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
					printf(1,"               Ia: %6.4f p.u. @ %6.1f Deg., ",{cabs(ia),arg(ia)})
					printf(1,"Ib: %6.4f p.u. @ %6.1f Deg., Ic: %6.4f p.u. @ %6.1f Deg.\n\n",{cabs(ib),arg(ib),cabs(ic),arg(ic)})
					printf(fno,"Fault Duty Contribution: %10.1f MVA, Ia0: %6.4f p.u. @ %6.1f Deg., ",{fcon,cabs(ia0),arg(ia0)})
					printf(fno,"Ia1: %6.4f p.u. @ %6.1f Deg., Ia2: %6.4f p.u. @ %6.1f Deg.\n",{cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
					printf(fno,"                 Ia: %6.4f p.u. @ %6.1f Deg., ",{cabs(ia),arg(ia)})
					printf(fno,"Ib: %6.4f p.u. @ %6.1f Deg., Ic: %6.4f p.u. @ %6.1f Deg.\n\n",{cabs(ib),arg(ib),cabs(ic),arg(ic)})
				end if
			end  for
		end if
	end for
end procedure

integer frombus,tobus,fni,fno,ti,nbus,foundbus,ii,ifaultedbus,faultedbus,nline,conn_type
sequence filename,ifilename,ofilename,title,zbus1,zbus2,zbus0,bus,message,tf,tpf,slgf,abc,dlgf,llf
sequence zth1,zth2,zth0,ia0,ia1,ia2,ia,ib,ic,i012,iabc,va,vb,vc,vabc,rawline,v012,va0,va1,va2,z02
atom r1,x1,r2,x2,r0,x0,bmva,phase_shift
text_color(YELLOW)
bk_color(BLUE)
clear_screen()
cunity={1.0,0.0}
v=cunity
a={cos(deg_rad(120.0)),sin(deg_rad(120.0))}
a2=cpower(a,2.0)
amatrix={{cunity,cunity,cunity},{cunity,a2,a},{cunity,a,a2}}
printf(1,"Short Circuit Program - Gerardo V. Lozada\n\n",{})
printf(1,"Enter Dataset Filename (No Extension): ",{})
filename=getstr(0)
ifilename=filename & ".dat"
ofilename=filename & ".out"
fni=open(ifilename,"r")
fno=open(ofilename,"w")
printf(fno,"Short Circuit Program - Gerardo V. Lozada\n\n",{})
printf(fno,"Input Data Filename: %s\n\n",{upper(ifilename)})
title=getstr(fni)
printf(1,"Case Title: %s\n\n",{title})
printf(fno,"Case Title: %s\n\n",{title})
bmva=input(fni)
printf(1,"Base MVA: %10.2f\n",{bmva})
printf(fno,"Base MVA: %10.2f\n",{bmva})
frombus=1
bus={}
zbus1={}
zbus2={}
zbus0={}
rawline={}
while frombus>=0 do
	frombus=input(fni)
	if frombus>=0 then
		tobus=input(fni)
		r1=input(fni)
		x1=input(fni)
		r2=input(fni)
		x2=input(fni)
		r0=input(fni)
		x0=input(fni)
		conn_type=input(fni)
		phase_shift=input(fni)
		rawline=rawline&{{{r1,x1},{r2,x2},{r0,x0},frombus,tobus,conn_type,phase_shift}}
	end if
end while
nline=length(rawline)
for i=1 to nline do
	r1=rawline[i][1][1]
	x1=rawline[i][1][2]
	r2=rawline[i][2][1]
	x2=rawline[i][2][2]
	r0=rawline[i][3][1]
	x0=rawline[i][3][2]
	frombus=rawline[i][4]
	tobus=rawline[i][5]
	conn_type=rawline[i][6]
	if (newbus(bus,frombus)=1) or (newbus(bus,tobus)=1) then
		if (newbus(bus,frombus)=1) and (tobus=0) then
			bus=addbus(bus,frombus)
			zbus1=addbustype1(zbus1,r1,x1)
			zbus2=addbustype1(zbus2,r2,x2)
			zbus0=addbustype1(zbus0,r0,x0)
		else
			if (newbus(bus,tobus)=1) and (frombus=0) then
				bus=addbus(bus,tobus)
				zbus1=addbustype1(zbus1,r1,x1)
				zbus2=addbustype1(zbus2,r2,x2)
				zbus0=addbustype1(zbus0,r0,x0)
			else
				if (newbus(bus,frombus)=1) and (newbus(bus,tobus)=0) then
					bus=addbus(bus,frombus)
					zbus1=addbustype2(zbus1,bus,tobus,r1,x1)
					zbus2=addbustype2(zbus2,bus,tobus,r2,x2)
					if (conn_type<3) then
						if (conn_type=0) then
							zbus0=addbustype2(zbus0,bus,tobus,r0,x0)
						else
							if (conn_type=2) then
								zbus0=addbustype2(zbus0,bus,frombus,0.0,0.0)
								zbus0=addbustype3(zbus0,bus,tobus,r0,x0)
							else
								if (conn_type=1) then
									zbus0=addbustype1(zbus0,r0,x0)
								end if
							end if
						end if
					end if
				else
					if (newbus(bus,tobus)=1) and (newbus(bus,frombus)=0) then
						bus=addbus(bus,tobus)
						zbus1=addbustype2(zbus1,bus,frombus,r1,x1)
						zbus2=addbustype2(zbus2,bus,frombus,r2,x2)
						if (conn_type<3) then
							if (conn_type=0) then								
								zbus0=addbustype2(zbus0,bus,frombus,r0,x0)
							else
								if (conn_type=1) then
									zbus0=addbustype2(zbus0,bus,tobus,0.0,0.0)
									zbus0=addbustype3(zbus0,bus,frombus,r0,x0)
								else
									if (conn_type=2) then
										zbus0=addbustype1(zbus0,r0,x0)
									end if
								end if
							end if
						end if
					end if
				end if
			end if
		end if
	else
		if (newbus(bus,frombus)=0) and (tobus=0) then
			zbus1=addbustype3(zbus1,bus,frombus,r1,x1)
			zbus2=addbustype3(zbus2,bus,frombus,r2,x2)
			zbus0=addbustype3(zbus0,bus,frombus,r0,x0)
		else
			if (newbus(bus,tobus)=0) and (frombus=0) then
				zbus1=addbustype3(zbus1,bus,tobus,r1,x1)
				zbus2=addbustype3(zbus2,bus,tobus,r2,x2)
				zbus0=addbustype3(zbus0,bus,tobus,r0,x0)
			else
				if (newbus(bus,frombus)=0) and (newbus(bus,tobus)=0) then
					zbus1=addbustype4(zbus1,bus,frombus,tobus,r1,x1)
					zbus2=addbustype4(zbus2,bus,frombus,tobus,r2,x2)
					if (conn_type<3) then
						if (conn_type=0) then
							zbus0=addbustype4(zbus0,bus,frombus,tobus,r0,x0)
						else
							if (conn_type=1) then
								zbus0=addbustype3(zbus0,bus,frombus,r0,x0)
							else
								if (conn_type=2) then
									zbus0=addbustype3(zbus0,bus,tobus,r0,x0)
								end if
							end if
						end if
					end if
				end if
			end if
		end if
	end if
	message=sprintf("Z1 Bus Matrix after addition of line from %d to %d, r1=%11.4e and x1=%11.4e",{frombus,tobus,r1,x1})
	print_matrix(zbus1,message,fno)
	message=sprintf("Z2 Bus Matrix after addition of line from %d to %d, r2=%11.4e and x2=%11.4e",{frombus,tobus,r2,x2})
	print_matrix(zbus2,message,fno)
	message=sprintf("Z0 Bus Matrix after addition of line from %d to %d, r0=%11.4e and x0=%11.4e",{frombus,tobus,r0,x0})
	print_matrix(zbus0,message,fno)
	printf(1,"Bus Number Sequence After Line Addition From %d To %d: ",{frombus,tobus})
	printf(fno,"Bus Number Sequence After Line Addition From %d To %d: ",{frombus,tobus})
	for j=1 to length(bus) do
		printf(1,"%5d  ",{bus[j]})
		printf(fno,"%5d  ",{bus[j]})
	end for
	printf(1,"\n\n",{})
	printf(fno,"\n\n",{})
end for
close(fni)
nbus=length(bus)
for i=1 to nbus do
	for j=i to nbus do
		if bus[j]<bus[i] then
			ti=bus[i]
			bus[i]=bus[j]
			bus[j]=ti
			for k=1 to nbus do
				tf=zbus1[i][k]
				zbus1[i][k]=zbus1[j][k]
				zbus1[j][k]=tf
				tf=zbus2[i][k]
				zbus2[i][k]=zbus2[j][k]
				zbus2[j][k]=tf
				tf=zbus0[i][k]
				zbus0[i][k]=zbus0[j][k]
				zbus0[j][k]=tf
			end for
			for k=1 to nbus do
				tf=zbus1[k][i]
				zbus1[k][i]=zbus1[k][j]
				zbus1[k][j]=tf
				tf=zbus2[k][i]
				zbus2[k][i]=zbus2[k][j]
				zbus2[k][j]=tf
				tf=zbus0[k][i]
				zbus0[k][i]=zbus0[k][j]
				zbus0[k][j]=tf
			end for
		end if
	end for
end for
printf(1,"\nFinal Bus Number Sequence After Sorting: ",{})
printf(fno,"\nFinal Bus Number Sequence After Sorting: ",{})
for i=1 to nbus do
	printf(1,"%5d  ",{bus[i]})
	printf(fno,"%5d  ",{bus[i]})
end for
printf(1,"\n\n",{})
printf(fno,"\n\n",{})
printf(1,"Raw Line Impedance Data:\nFrom Bus   To Bus                 Z1 (p.u.)                 Z2 (p.u.)                 Z0 (p.u.)   Connection\n",{})
printf(fno,"Raw Line Impedance Data:\nFrom Bus   To Bus                 Z1 (p.u.)                 Z2 (p.u.)                 Z0 (p.u.)   Connection\n",{})
for i=1 to nline do
	printf(1,"%8d   %6d   %10.4f +j%10.4f",{rawline[i][4],rawline[i][5],real(rawline[i][1]),imag(rawline[i][1])})
	printf(1,"   %10.4f +j%10.4f   %10.4f +j%10.4f   %10d\n",{real(rawline[i][2]),imag(rawline[i][2]),real(rawline[i][3]),imag(rawline[i][3]),rawline[i][6]})
	printf(fno,"%8d   %6d   %10.4f +j%10.4f",{rawline[i][4],rawline[i][5],real(rawline[i][1]),imag(rawline[i][1])})
	printf(fno,"   %10.4f +j%10.4f   %10.4f +j%10.4f   %10d\n",{real(rawline[i][2]),imag(rawline[i][2]),real(rawline[i][3]),imag(rawline[i][3]),rawline[i][6]})
end for
print_matrix(zbus1,"Final Z1 Bus Matrix",fno)
print_matrix(zbus2,"Final Z2 Bus Matrix",fno)
print_matrix(zbus0,"Final Z0 Bus Matrix",fno)
faultedbus=1
while (faultedbus>0) do
	printf(1,"\n\nBus Number Sequence is ",{})
	for i=1 to nbus do
		printf(1,"%5d  ",{bus[i]})
	end for
	printf(1,"\n\nBus Number To Simulate Fault (0 to end): ",{})
	faultedbus=input(0)
	if (faultedbus>0) then
		printf(fno,"\n\nBus Number To Simulate Fault: %d\n\n",{faultedbus})
		foundbus=0
		ii=0
		while (foundbus=0) and (ii<nbus) do
			ii+=1
			if (bus[ii]=faultedbus) then
				foundbus=1
				ifaultedbus=ii
			end if
		end while
		if (foundbus=1) then
			printf(1,"P.U. Thevenin Impedances at Faulted Bus %d:\n",{bus[ifaultedbus]})
			printf(fno,"P.U. Thevenin Impedances at Faulted Bus %d:\n",{bus[ifaultedbus]})
			zth1=zbus1[ifaultedbus][ifaultedbus]
			zth2=zbus2[ifaultedbus][ifaultedbus]
			zth0=zbus0[ifaultedbus][ifaultedbus]
			printf(1,"Z1: %10.4f+j%10.4f, Z2:%10.4f+j%10.4f & Z0:%10.4f+j%10.4f\n",{real(zth1),imag(zth1),real(zth2),imag(zth2),real(zth0),imag(zth0)})
			printf(fno,"Z1: %10.4f+j%10.4f, Z2:%10.4f+j%10.4f & Z0:%10.4f+j%10.4f\n",{real(zth1),imag(zth1),real(zth2),imag(zth2),real(zth0),imag(zth0)})
			ia0=cinit
			ia1=cdiv(v,zbus1[ifaultedbus][ifaultedbus])
			ia2=cinit
			i012={{ia0},{ia1},{ia2}}
			abc=abc_current_voltage(i012,zbus1,zbus2,zbus0,v,ifaultedbus)
			iabc=abc[1]
			ia=iabc[1][1]
			ib=iabc[2][1]
			ic=iabc[3][1]
			v012=abc[2]
			va0=v012[1][1]
			va1=v012[2][1]
			va2=v012[3][1]
			vabc=abc[3]
			va=vabc[1][1]
			vb=vabc[2][1]
			vc=vabc[3][1]
			tpf=cmult(v,conj(ia))
			printf(1,"\n3-Phase MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*tpf),arg(tpf)})
			printf(1,"P.U. Sequence Currents a0: %6.4f@%6.1f Deg., a1: %6.4f@%6.1f Deg. & a2: %6.4f@%6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(1,"P.U. Phase Currents A: %6.4f@%6.1f Deg., B: %6.4f@%6.1f Deg. & C: %6.4f@%6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(1,"P.U. Sequence Voltages a0: %6.4f@%6.1f Deg., a1: %6.4f@%6.1f Deg. & a2: %6.4f@%6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(1,"P.U. Phase Voltages A: %6.4f@%6.1f Deg., B: %6.4f@%6.1f Deg. & C: %6.4f@%6.1f Deg.\n\n",{cabs(va),arg(va),cabs(vb),arg(vb),cabs(vc),arg(vc)})
			printf(fno,"\n3-Phase MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*tpf),arg(tpf)})
			printf(fno,"P.U. Sequence Currents a0: %6.4f@%6.1f Deg., a1: %6.4f@%6.1f Deg. & a2: %6.4f@%6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(fno,"P.U. Phase Currents A: %6.4f@%6.1f Deg., B: %6.4f@%6.1f Deg. & C: %6.4f@%6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(fno,"P.U. Sequence Voltages a0: %6.4f@%6.1f Deg., a1: %6.4f@%6.1f Deg. & a2: %6.4f@%6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"P.U. Phase Voltages A: %6.4f@%6.1f Deg., B: %6.4f@%6.1f Deg. & C: %6.4f@%6.1f Deg.\n\n",{cabs(va),arg(va),cabs(vb),arg(vb),cabs(vc),arg(vc)})
			fault_contributions(iabc,vabc,v,zbus1,zbus2,zbus0,bus,rawline,ifaultedbus,fno,bmva)
			ia0=cdiv(v,zbus1[ifaultedbus][ifaultedbus]+zbus2[ifaultedbus][ifaultedbus]+zbus0[ifaultedbus][ifaultedbus])
			ia1=ia0
			ia2=ia0
			i012={{ia0},{ia1},{ia2}}
			abc=abc_current_voltage(i012,zbus1,zbus2,zbus0,v,ifaultedbus)
			iabc=abc[1]
			ia=iabc[1][1]
			ib=iabc[2][1]
			ic=iabc[3][1]
			v012=abc[2]
			va0=v012[1][1]
			va1=v012[2][1]
			va2=v012[3][1]
			vabc=abc[3]
			va=vabc[1][1]
			vb=vabc[2][1]
			vc=vabc[3][1]
			slgf=cmult(v,conj(ia))
			printf(1,"\nSingle-Line-To-Ground MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*slgf),arg(slgf)})
			printf(1,"P.U. Sequence Currents a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(1,"P.U. Phase Currents A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(1,"P.U. Sequence Voltages a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(1,"P.U. Phase Voltages A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"\nSingle-Line-To-Ground MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*slgf),arg(slgf)})
			printf(fno,"P.U. Sequence Currents a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(fno,"P.U. Phase Currents A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(fno,"P.U. Sequence Voltages a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"P.U. Phase Voltages A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(va),arg(va),cabs(vb),arg(vb),cabs(vc),arg(vc)})
			fault_contributions(iabc,vabc,v,zbus1,zbus2,zbus0,bus,rawline,ifaultedbus,fno,bmva)
			ia1=cdiv(v,zbus1[ifaultedbus][ifaultedbus]+zbus2[ifaultedbus][ifaultedbus])
			ia2=-ia1
			ia0=cinit
			i012={{ia0},{ia1},{ia2}}
			abc=abc_current_voltage(i012,zbus1,zbus2,zbus0,v,ifaultedbus)
			iabc=abc[1]
			ia=iabc[1][1]
			ib=iabc[2][1]
			ic=iabc[3][1]
			v012=abc[2]
			va0=v012[1][1]
			va1=v012[2][1]
			va2=v012[3][1]
			vabc=abc[3]
			va=vabc[1][1]
			vb=vabc[2][1]
			vc=vabc[3][1]
			llf=cmult(v,conj(ib))
			printf(1,"\nLine-To-Line MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*llf),arg(llf)})
			printf(1,"P.U. Sequence Currents a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(1,"P.U. Phase Currents A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(1,"P.U. Sequence Voltages a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(1,"P.U. Phase Voltages A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"\nLine-To-Line MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*llf),arg(llf)})
			printf(fno,"P.U. Sequence Currents a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(fno,"P.U. Phase Currents A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(fno,"P.U. Sequence Voltages a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"P.U. Phase Voltages A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(va),arg(va),cabs(vb),arg(vb),cabs(vc),arg(vc)})
			fault_contributions(iabc,vabc,v,zbus1,zbus2,zbus0,bus,rawline,ifaultedbus,fno,bmva)
			z02=cdiv(cmult(zbus2[ifaultedbus][ifaultedbus],zbus0[ifaultedbus][ifaultedbus]),zbus2[ifaultedbus][ifaultedbus]+zbus0[ifaultedbus][ifaultedbus])
			ia1=cdiv(v,zbus1[ifaultedbus][ifaultedbus]+z02)
			ia0=-cmult(ia1,cdiv(zbus2[ifaultedbus][ifaultedbus],zbus2[ifaultedbus][ifaultedbus]+zbus0[ifaultedbus][ifaultedbus]))
			ia2=-cmult(ia1,cdiv(zbus0[ifaultedbus][ifaultedbus],zbus2[ifaultedbus][ifaultedbus]+zbus0[ifaultedbus][ifaultedbus]))
			i012={{ia0},{ia1},{ia2}}
			abc=abc_current_voltage(i012,zbus1,zbus2,zbus0,v,ifaultedbus)
			iabc=abc[1]
			ia=iabc[1][1]
			ib=iabc[2][1]
			ic=iabc[3][1]
			v012=abc[2]
			va0=v012[1][1]
			va1=v012[2][1]
			va2=v012[3][1]
			vabc=abc[3]
			va=vabc[1][1]
			vb=vabc[2][1]
			vc=vabc[3][1]
			dlgf=cmult(v,conj(ib))
			printf(1,"\nDouble-Line-To-Ground MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*dlgf),arg(dlgf)})
			printf(1,"P.U. Sequence Currents a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(1,"P.U. Phase Currents A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(1,"P.U. Sequence Voltages a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(1,"P.U. Phase Voltages A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"\nDouble-Line-To-Ground MVA Fault Duty = %10.2f at %8.2f Degrees\n\n",{cabs(bmva*dlgf),arg(dlgf)})
			printf(fno,"P.U. Sequence Currents a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(ia0),arg(ia0),cabs(ia1),arg(ia1),cabs(ia2),arg(ia2)})
			printf(fno,"P.U. Phase Currents A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(ia),arg(ia),cabs(ib),arg(ib),cabs(ic),arg(ic)})
			printf(fno,"P.U. Sequence Voltages a0: %6.4f @ %6.1f Deg., a1: %6.4f @ %6.1f Deg. & a2: %6.4f @ %6.1f Deg.\n",{cabs(va0),arg(va0),cabs(va1),arg(va1),cabs(va2),arg(va2)})
			printf(fno,"P.U. Phase Voltages A: %6.4f @ %6.1f Deg., B: %6.4f @ %6.1f Deg. & C: %6.4f @ %6.1f Deg.\n\n",{cabs(va),arg(va),cabs(vb),arg(vb),cabs(vc),arg(vc)})
			fault_contributions(iabc,vabc,v,zbus1,zbus2,zbus0,bus,rawline,ifaultedbus,fno,bmva)
		else
			printf(1,"Bus %d not found\n",{faultedbus})
			printf(fno,"Bus %d not found\n",{faultedbus})
		end if
	end if
end while
printf(fno,"\nProgram execution completed\n",{})
close(fno)
printf(1,"\nProgram execution completed\n",{})
